@@ -12,14 +12,16 @@
 #include <unistd.h>
 
 linyaps_box::container_ref::container_ref(const status_directory &status_dir, std::string id)
-    : id(std::move(id))
+    : id_(std::move(id))
     , status_dir_(status_dir)
 {
 }
 
+linyaps_box::container_ref::~container_ref() noexcept = default;
+
 linyaps_box::container_status_t linyaps_box::container_ref::status() const
 {
-    return this->status_dir_.read(this->id);
+    return this->status_dir_.read(this->id_);
 }
 
 void linyaps_box::container_ref::kill(int signal) const
@@ -111,5 +113,5 @@ const linyaps_box::status_directory &linyaps_box::container_ref::status_dir() co
 
 const std::string &linyaps_box::container_ref::get_id() const
 {
-    return this->id;
+    return this->id_;
 }
