@@ -9,11 +9,13 @@
 
 #include <fcntl.h>
 
-linyaps_box::utils::file_descriptor linyaps_box::utils::touch(const file_descriptor &root,
-                                                              const std::filesystem::path &path)
+auto linyaps_box::utils::touch(const file_descriptor &root,
+                               const std::filesystem::path &path,
+                               int flag,
+                               mode_t mode) -> linyaps_box::utils::file_descriptor
 {
     LINYAPS_BOX_DEBUG() << "touch " << path << " at " << inspect_fd(root.get());
-    int fd = ::openat(root.get(), path.c_str(), O_CREAT | O_WRONLY, 0666);
+    const auto fd = ::openat(root.get(), path.c_str(), flag, mode);
     if (fd == -1) {
         throw std::system_error(errno,
                                 std::system_category(),
